classdef Box < MSS.TreeNode
   
   
   
   properties (SetAccess=protected)
      
      client
      
      top = 0
      right = 0
      bottom = 0
      left = 0
      usedTop = 0
      usedRight = 0
      usedBottom = 0
      usedLeft = 0
      
      width = 0
      height = 0
      usedWidth = 0
      usedHeight = 0
      minWidth = 0
      maxWidth = 0
      minHeight = 0
      maxHeight = 0
      
      padding = [0 0 0 0]
      usedPadding  = [0 0 0 0]
      
      borderColor = {'transparent','transparent','transparent','transparent'}
      borderStyle = {'-','-','-','-'}
      borderWidth = [0 0 0 0]
      usedBorderWidth  = [0 0 0 0]
      
      margin = {0,0,0,0}
      usedMargin = [0 0 0 0]
      
      displayMode = 'block'
      positionMode = 'static'
      
   end % properties
   
   
   
   methods
      
      
      function obj = Box(client)
         
         if nargin > 0
            obj.client = client;
         end
         
         %          bgColor = get(gcf,'Color');
         %          obj.canvasAxes = builtin('axes',...
         %                'Color',bgColor,...
         %                'NextPlot','add',...
         %                'Position',[0 0 1 1],...
         %                'XColor',bgColor,...
         %                'YColor',bgColor,...
         %                'XLim',[0 1],...
         %                'YLim',[0 1],...
         %                'XTick',[],...
         %                'YTick',[]...
         %             );
         
      end % function
      
      
      function set(obj,property,value)
         
         assert(ischar(property),'Property name must be a string')
         
         switch lower(property)
            
            case 'display'
               
               assert( ismember(value,{'block','inline-block'}),...
                  'Invalid value for display property.' );
               obj.displayMode = value;
               
            case {'width','height'}
               
               value = MSS.Converter.standardizeLengths(value);
               assert( isscalar(value), 'Dimension %s must be a scalar', property );
               if ischar( value{1} )
                  obj.(lower(property)) = value{1};
               else
                  assert( value{1} >= 0 , 'Dimension %s must be nonnegative', property );
                  obj.(lower(property)) = value{1};
               end
               
            case 'padding'
               
               values = MSS.Converter.standardizeLengths(value);
               switch length(values)
                  case 1
                     set(obj,'PaddingTop',values{1});
                     set(obj,'PaddingRight',values{1});
                     set(obj,'PaddingBottom',values{1});
                     set(obj,'PaddingLeft',values{1});
                  case 2
                     set(obj,'PaddingTop',values{1});
                     set(obj,'PaddingRight',values{2});
                     set(obj,'PaddingBottom',values{1});
                     set(obj,'PaddingLeft',values{2});
                  case 3
                     set(obj,'PaddingTop',values{1});
                     set(obj,'PaddingRight',values{2});
                     set(obj,'PaddingBottom',values{3});
                     set(obj,'PaddingLeft',values{2});
                  case 4
                     set(obj,'PaddingTop',values{1});
                     set(obj,'PaddingRight',values{2});
                     set(obj,'PaddingBottom',values{3});
                     set(obj,'PaddingLeft',values{4});
               end
               
            case 'borderwidth'
               
               values = MSS.Converter.standardizeLengths(value);
               switch length(values)
                  case 1
                     set(obj,'BorderTopWidth',values{1});
                     set(obj,'BorderRightWidth',values{1});
                     set(obj,'BorderBottomWidth',values{1});
                     set(obj,'BorderLeftWidth',values{1});
                  case 2
                     set(obj,'BorderTopWidth',values{1});
                     set(obj,'BorderRightWidth',values{2});
                     set(obj,'BorderBottomWidth',values{1});
                     set(obj,'BorderLeftWidth',values{2});
                  case 3
                     set(obj,'BorderTopWidth',values{1});
                     set(obj,'BorderRightWidth',values{2});
                     set(obj,'BorderBottomWidth',values{3});
                     set(obj,'BorderLeftWidth',values{2});
                  case 4
                     set(obj,'BorderTopWidth',values{1});
                     set(obj,'BorderRightWidth',values{2});
                     set(obj,'BorderBottomWidth',values{3});
                     set(obj,'BorderLeftWidth',values{4});
               end
               
            case {'paddingtop','paddingright','paddingbottom','paddingleft'...
                  'bordertopwidth','borderrightwidth','borderbottomwidth','borderleftwidth'}
               
               value = MSS.Converter.standardizeLengths(value);
               if not(isempty(value))
                  assert( isscalar(value), 'Dimension %s must be a scalar', property );
                  assert( value{1} >= 0 , 'Dimension %s must be nonnegative', property );
                  assert( not(isequal(value{1},'auto')),'Property %s cannot be ''auto''', property );
                  switch lower(property)
                     case 'paddingtop'
                        obj.padding(1) = value{1};
                     case 'paddingright'
                        obj.padding(2) = value{1};
                     case 'paddingbottom'
                        obj.padding(3) = value{1};
                     case 'paddingleft'
                        obj.padding(4) = value{1};
                     case 'bordertopwidth'
                        obj.borderWidth(1) = value{1};
                     case 'borderrightwidth'
                        obj.borderWidth(2) = value{1};
                     case 'borderbottomwidth'
                        obj.borderWidth(3) = value{1};
                     case 'borderleftwidth'
                        obj.borderWidth(4) = value{1};
                  end
               end
               
            case 'margin'
               
               values = MSS.Converter.standardizeLengths(value);
               switch length(values)
                  case 1
                     set(obj,'MarginTop',values{1});
                     set(obj,'MarginRight',values{1});
                     set(obj,'MarginBottom',values{1});
                     set(obj,'MarginLeft',values{1});
                  case 2
                     set(obj,'MarginTop',values{1});
                     set(obj,'MarginRight',values{2});
                     set(obj,'MarginBottom',values{1});
                     set(obj,'MarginLeft',values{2});
                  case 3
                     set(obj,'MarginTop',values{1});
                     set(obj,'MarginRight',values{2});
                     set(obj,'MarginBottom',values{3});
                     set(obj,'MarginLeft',values{2});
                  case 4
                     set(obj,'MarginTop',values{1});
                     set(obj,'MarginRight',values{2});
                     set(obj,'MarginBottom',values{3});
                     set(obj,'MarginLeft',values{4});
               end
               
            case {'margintop','marginright','marginbottom','marginleft'}
               
               value = MSS.Converter.standardizeLengths(value);
               if not(isempty(value))
                  assert( isscalar(value), 'Dimension %s must be a scalar', property );
                  switch lower(property)
                     case 'margintop'
                        obj.margin{1} = value{1};
                     case 'marginright'
                        obj.margin{2} = value{1};
                     case 'marginbottom'
                        obj.margin{3} = value{1};
                     case 'marginleft'
                        obj.margin{4} = value{1};
                  end
               end
               
               
            case 'bordercolor'
               
               values = MSS.Converter.standardizeColors(value);
               switch length(values)
                  case 1
                     set(obj,'BorderTopColor',values{1});
                     set(obj,'BorderRightColor',values{1});
                     set(obj,'BorderBottomColor',values{1});
                     set(obj,'BorderLeftColor',values{1});
                  case 2
                     set(obj,'BorderTopColor',values{1});
                     set(obj,'BorderRightColor',values{2});
                     set(obj,'BorderBottomColor',values{1});
                     set(obj,'BorderLeftColor',values{2});
                  case 3
                     set(obj,'BorderTopColor',values{1});
                     set(obj,'BorderRightColor',values{2});
                     set(obj,'BorderBottomColor',values{3});
                     set(obj,'BorderLeftColor',values{2});
                  case 4
                     set(obj,'BorderTopColor',values{1});
                     set(obj,'BorderRightColor',values{2});
                     set(obj,'BorderBottomColor',values{3});
                     set(obj,'BorderLeftColor',values{4});
               end
               
               
            case {'bordertopcolor','borderrightcolor','borderbottomcolor','borderleftcolor'}
               
               value = MSS.Converter.standardizeColors(value);
               if not(isempty(value))
                  assert( isscalar(value), 'Must specify only one color for %s', property )
                  if isequal(value{1},'none')
                     value{1} = 'transparent';
                  end
                  switch lower(property)
                     case 'bordertopcolor'
                        obj.borderColor{1} = value{1};
                     case 'borderrightcolor'
                        obj.borderColor{2} = value{1};
                     case 'borderbottomcolor'
                        obj.borderColor{3} = value{1};
                     case 'borderleftcolor'
                        obj.borderColor{4} = value{1};
                  end
               end
               
            case 'borderstyle'
               
               assert(ischar(value),'Line style must be a string');
               values = regexp(value,'(?<=\s*)\S+(?=\s*)','match');
               switch length(values)
                  case 1
                     set(obj,'BorderTopStyle',values{1});
                     set(obj,'BorderRightStyle',values{1});
                     set(obj,'BorderBottomStyle',values{1});
                     set(obj,'BorderLeftStyle',values{1});
                  case 2
                     set(obj,'BorderTopStyle',values{1});
                     set(obj,'BorderRightStyle',values{2});
                     set(obj,'BorderBottomStyle',values{1});
                     set(obj,'BorderLeftStyle',values{2});
                  case 3
                     set(obj,'BorderTopStyle',values{1});
                     set(obj,'BorderRightStyle',values{2});
                     set(obj,'BorderBottomStyle',values{3});
                     set(obj,'BorderLeftStyle',values{2});
                  case 4
                     set(obj,'BorderTopStyle',values{1});
                     set(obj,'BorderRightStyle',values{2});
                     set(obj,'BorderBottomStyle',values{3});
                     set(obj,'BorderLeftStyle',values{4});
               end
               
            case {'bordertopstyle','borderrightstyle','borderbottomstyle','borderleftstyle'}
               
               assert(ischar(value),'Line style must be a string');
               
               switch lower(value)
                  case {'-','solid','double','groove','ridge','inset','outset'}
                     value = '-';
                  case {'--','dashed'}
                     value = '--';
                  case {':','dotted'}
                     value = ':';
                  case {'none','hidden'}
                     value = '';
                  otherwise
                     error('Invalid line style');
               end
               
               switch lower(property)
                  case 'bordertopstyle'
                     obj.borderStyle{1} = value;
                  case 'borderrightstyle'
                     obj.borderStyle{2} = value;
                  case 'borderbottomstyle'
                     obj.borderStyle{3} = value;
                  case 'borderleftstyle'
                     obj.borderStyle{4} = value;
               end
               
            otherwise
               
               throw(MException('MSS:Box:badProperty','Invalid property ''%s''',property))
               
         end % switch
         
      end % function set
      
      
      function value = get(obj,property)
         
         assert( ischar(property), 'Property name must be a string' );
         
         props = properties(obj);
         
         matches = strncmpi( property, props, length(property) );
         
         if sum(matches) == 1
            value = obj.(props{matches});
         elseif sum(matches) > 1
            error( 'Ambiguous property name: %s', property );
         else
            switch lower(property)
               case 'paddingtop'
                  value = obj.padding(1);
               case 'paddingright'
                  value = obj.padding(2);
               case 'paddingbottom'
                  value = obj.padding(3);
               case 'paddingleft'
                  value = obj.padding(4);
               case 'bordertopcolor'
                  value = obj.borderColor{1};
               case 'borderrightcolor'
                  value = obj.borderColor{2};
               case 'borderbottomcolor'
                  value = obj.borderColor{3};
               case 'borderleftcolor'
                  value = obj.borderColor{4};
               case 'bordertopstyle'
                  value = obj.borderStyle{1};
               case 'borderrightstyle'
                  value = obj.borderStyle{2};
               case 'borderbottomstyle'
                  value = obj.borderStyle{3};
               case 'borderleftstyle'
                  value = obj.borderStyle{4};
               case 'bordertopwidth'
                  value = obj.borderWidth(1);
               case 'borderrightwidth'
                  value = obj.borderWidth(2);
               case 'borderbottomwidth'
                  value = obj.borderWidth(3);
               case 'borderleftwidth'
                  value = obj.borderWidth(4);
               case 'margintop'
                  value = obj.margin{1};
               case 'marginright'
                  value = obj.margin{2};
               case 'marginbottom'
                  value = obj.margin{3};
               case 'marginleft'
                  value = obj.margin{4};
               otherwise
                  error( 'Invalid property name: %s', property );
            end
         end

         
      end % function get
      
      
      function reply = hasAuto(obj)
         
         reply = isequal(obj.width,'auto') || ...
                 isequal(obj.margin{2},'auto') || ...
                 isequal(obj.margin{4},'auto');
              
         if not(reply)
            for k = 1:obj.numChildren();
               reply = reply || obj.children{k}.hasAuto();
            end
         end
         
      end
      
      
      function reply = hasAutoHeight(obj)
         
         reply = isequal(obj.height,'auto') || ...
            isequal(obj.margin{1},'auto') || ...
            isequal(obj.margin{3},'auto');
         
         if not(reply)
            for k = 1:obj.numChildren();
               reply = reply || obj.children{k}.hasAutoHeight();
            end
         end
      end % function
      
      
      function removeLines(obj)
         
         kids = obj.children;
         for j = 1:length(kids)
            if isa(kids{j},'MSS.LineBox')
               obj.removeGeneration(kids{j});
            end
         end % for j
         
      end % function
      
      
      function insertLines(obj)
         
         % Recurse down the tree
         mode = 'block';
         lines = {};
         for j = 1:obj.numChildren()
            
            kid = obj.children{j};
            kid.insertLines();
            
            if isequal(mode,'block')
               if isequal(kid.displayMode,'inline-block')
                  left = j;
                  mode = 'inline';
               end
            else
               if not(isequal(kid.displayMode,'inline-block'))
                  right = j-1;
                  lines{end+1} = [left,right];
                  mode = 'block';
               elseif (j == obj.numChildren())
                  right = j;
                  lines{end+1} = [left,right];
               end
            end
            
         end % for j
         
         lost = 0;
         for k = 1:length(lines)
            
            line = lines{k};
            lineBox = MSS.LineBox();
            obj.insertGeneration(lineBox,line(1)-lost,line(2)-lost);
            lineBox.findBreaks();
            
            lost = line(2)-line(1);
            
         end % for k
         
      end % function
      
      
      function layoutChildren(obj)

         extraHeight = max(0,obj.usedHeight-obj.minHeight);
         n = 0;
         for k = 1:obj.numChildren()
            obj.children{k}.maxWidth = obj.usedWidth;
            n = n + obj.children{k}.countVertAuto();
         end
         
         if (n == 0)
            auto = 0;
         else
            auto = extraHeight/n;
         end
         
         for k = 1:obj.numChildren()
            obj.children{k}.maxHeight = obj.children{k}.minHeight + auto*obj.children{k}.countVertAuto();
            obj.children{k}.layout();
         end
         
      end % function
      
      
      function computeLocation(obj)
         
         % Compute distance to top border
         prev = obj.getPrevSibling();
         if not(isempty(prev))
            if isequal(obj.displayMode,'block')
               obj.usedTop = prev.usedTop - prev.usedMargin(1) + prev.maxHeight + obj.usedMargin(1);
            else
               j = obj.parent.getLineIndex(obj);
               k = obj.parent.getLineIndex(prev);
               if (j > k)
                  obj.usedTop = prev.usedTop - prev.usedMargin(1) + prev.maxHeight + obj.usedMargin(1);
               elseif (j==k)
                  obj.usedTop = prev.usedTop - prev.usedMargin(1) + obj.usedMargin(1);
               end
            end
         else
            obj.usedTop = obj.parent.usedTop + obj.parent.usedBorderWidth(1) ...
               + obj.parent.usedPadding(1) + obj.usedMargin(1);
         end

         
         % Compute distance to left border
         usePrevious = false;
         if isequal(obj.displayMode,'inline-block')
            
            if not(isempty(prev));
               j = obj.parent.getLineIndex(obj);
               k = obj.parent.getLineIndex(prev);
               if (j == k)
                  usePrevious = true;
               end
            end
         end
         if usePrevious
            obj.usedLeft = prev.usedLeft - prev.usedMargin(4) + prev.maxWidth + obj.usedMargin(4);
         else
            obj.usedLeft = obj.parent.usedLeft + obj.parent.usedBorderWidth(4) ...
               + obj.parent.usedPadding(4) + obj.usedMargin(4);
         end
         
      end % function
      
      
      function computeMinSize(obj)
         
         h = sum(obj.padding([1 3])) + sum(obj.borderWidth([1 3]));
         w = sum(obj.padding([2 4])) + sum(obj.borderWidth([2 4]));
         if not(isequal(obj.margin{1},'auto'))
            h = h + obj.margin{1};
         end
         if not(isequal(obj.margin{3},'auto'))
            h = h + obj.margin{3};
         end
         if not(isequal(obj.margin{2},'auto'))
            w = w + obj.margin{2};
         end
         if not(isequal(obj.margin{4},'auto'))
            w = w + obj.margin{4};
         end
         
         for k = 1:obj.numChildren()
            obj.children{k}.computeMinSize();
         end
         
         if not(isequal(obj.width,'auto'))
            w = w + obj.width;
         else
            for k = 1:obj.numChildren()
               w = w + obj.children{k}.minWidth;
            end
         end
         
         if not(isequal(obj.height,'auto'))
            h = h + obj.height;
         else
            for k = 1:obj.numChildren()
               h = h + obj.children{k}.minHeight;
            end
         end
         
         obj.minHeight = h;
         obj.minWidth = w;
         
      end % function
      
      
      function n = countVertAuto(obj)
         
         n = isequal(obj.height,'auto') || isequal(obj.margin{1},'auto') ...
            || isequal(obj.margin{3},'auto');
         n = double(n);
         
      end % function
      
      
      function n = countHorzAuto(obj)
         
         n = isequal(obj.width,'auto') || isequal(obj.margin{2},'auto') ...
            || isequal(obj.margin{4},'auto');
         n = double(n);
         
      end % function
      
      
      function computeMinHeight(obj)
         
         if isempty(obj.minHeight)
            obj.computeRelHeight();
            if obj.relHeight == 0
               obj.minHeight = 0;
            else
               obj.minHeight = real(obj.relHeight) + imag(obj.relHeight) + obj.parent.minHeight;
            end
         end
         
      end % function
      
      
   end % methods
   
   
   
end % classdef
