classdef FrameBox < MSS.Box
   

   methods
      
      
      function obj = FrameBox(varargin)
         
         obj = obj@MSS.Box(varargin{:});
         
         obj.displayMode = 'block';
         
      end
      
      
      function set(obj,property,value,varargin)
         
         set@MSS.Box(obj,property,value);
         
         if not(isempty(varargin))
            set(obj,varargin{:});
         end
         
      end % function
      
      
      function layout(obj)
         
         % Determine how much auto space we have
         autoW = obj.maxWidth - obj.minWidth;
         autoH = obj.maxHeight - obj.minHeight;
         
         % Compute padding and borders
         obj.usedPadding = obj.padding;
         obj.usedBorderWidth = obj.borderWidth;
         
         % Compute width
         if isequal(obj.width,'auto')
            obj.usedWidth = max(0,autoW);
            autoW = min(0,autoW);
         else
            obj.usedWidth = obj.width;
         end
         
         % Compute left margin
         if isequal(obj.margin{4},'auto')
            if isequal(obj.margin{2},'auto')
               obj.usedMargin(4) = max(0,autoW/2);
            else
               obj.usedMargin(4) = max(0,autoW);
            end
            autoW = autoW - obj.usedMargin(4);
         else
            obj.usedMargin(4) = obj.margin{4};
         end
         
         % Compute right margin
         if isequal(obj.margin{2},'auto')
            obj.usedMargin(2) = max(0,autoW);
         else
            obj.usedMargin(2) = min(obj.margin{2},max(0,autoW));
         end
         
         % Compute height
         if isequal(obj.height,'auto')
            obj.usedHeight = max(0,autoH);
            autoH = min(0,autoH);
         else
            obj.usedHeight = obj.height;
         end
         
         % Compute top margin
         if isequal(obj.margin{1},'auto')
            if isequal(obj.margin{3},'auto')
               obj.usedMargin(1) = max(0,autoH/2);
            else
               obj.usedMargin(1) = max(0,autoH);
            end
            autoH = autoH - obj.usedMargin(1);
         else
            obj.usedMargin(1) = obj.margin{1};
         end
         
         % Compute bottom margin
         if isequal(obj.margin{3},'auto')
            obj.usedMargin(3) = max(0,autoH);
         else
            obj.usedMargin(3) = min(obj.margin{3},max(0,autoH));
         end
         
         obj.computeLocation();

         obj.layoutChildren();
         
      end % function
      
      
      function computeRelWidth(obj)
         
         if not(isempty( obj.relWidth ))
            return;
         end
         
         rel = sum(obj.borderWidth([2,4])) + sum(obj.padding([2,4]));
         
         if isequal(obj.width,'auto')
            try
               rel = rel + obj.client.minWidth;
            catch
            end
         else
            rel = rel + obj.width;
         end
         
         if not(isequal(obj.margin{2},'auto'))
            rel = rel + obj.margin{2};
         end
         if not(isequal(obj.margin{4},'auto'))
            rel = rel + obj.margin{4};
         end
         
         if not(obj.isLeaf())
         
            childRel = 0;
            for k = 1:obj.numChildren()
               if isempty(obj.children{k}.relWidth)
                  obj.children{k}.computeRelWidth();
               end
               childRel = childRel + obj.children{k}.relWidth;
            end

            rel = (rel + real(childRel)) / (1 - imag(childRel));
            
         end
         
         obj.relWidth = rel;

      end % function
      
      
      function computeMinHeight(obj)
         obj.computeRelHeight();
         
         childRel = 0;
         for j = 1:obj.numChildren()
            
            if isa( obj.children{j}, 'MSS.FrameBox' )
               obj.children{j}.computeRelHeight();
               childRel = childRel + obj.children{j}.relHeight;
            end
            
         end
         
         
         for k = 1:obj.numChildren()
            if isa( obj.children{k}, 'MSS.LineBox' )
               % For each line of this line box
               for m = 1:obj.children{k}.numLines()
                  line = obj.children{k}.getLine(m);
                  % See which box in this line is tallest
                  best = 0;
                  for n = 1:length(line)
                     
                     line{n}.computeRelHeight();
                     tempChildRel = childRel + line{n}.relHeight;
                     tempRel = (obj.relHeight + real(tempChildRel)) / (1 - imag(tempChildRel));
                     tempVal = real(tempRel) + imag(tempRel) + obj.parent.minHeight;
                     if tempVal >= best
                        keep = n;
                     end
                  end
                  % Only the tallest block contributes
                  childRel = childRel + line{keep}.relHeight;
               end
            end   
         end % for k
         
         worstRel = (obj.relHeight + real(childRel))/(1-imag(childRel));

         if worstRel == 0
            obj.minHeight = 0;
         else
            if isequal( obj.displayMode, 'block' );
               obj.minHeight = real(worstRel) + imag(worstRel) * obj.parent.minHeight;
            else
               obj.minHeight = real(worstRel) + imag(worstRel) * obj.parent.parent.minHeight;
            end
         end
      end % function
      
      
      function computeRelHeight(obj)
         
         if not(isempty( obj.relHeight ))
            return;
         end
         
         rel = sum(obj.borderWidth([1,3])) + sum(obj.padding([1,3]));
         
         if isequal(obj.height,'auto')
            try
               rel = rel + obj.client.minHeight;
            catch
            end
         else
            rel = rel + obj.height;
         end
         
         if not(isequal(obj.margin{1},'auto'))
            rel = rel + obj.margin{1};
         end
         if not(isequal(obj.margin{3},'auto'))
            rel = rel + obj.margin{3};
         end
         
         obj.relHeight = rel;

      end % function
      
      
      function finalizeWidths(obj)
         
         w = obj.parent.usedWidth;
         for k = [2,4]
            if isreal(obj.padding(k))
               obj.usedPadding(k) = obj.padding(k);
            else
               obj.usedPadding(k) = imag(obj.padding(k)) * w;
            end
            if isreal(obj.borderWidth(k))
               obj.usedBorderWidth(k) = obj.borderWidth(k);
            else
               obj.usedBorderWidth(k) = imag(obj.borderWidth(k)) * w;
            end
         end
         
         remain = max( 0, w - sum(obj.usedPadding([2,4])) - sum(obj.usedBorderWidth([2,4])) );
         
         if isequal(obj.displayMode,'block')
            
            if isequal(obj.width,'auto')
               
               if isequal(obj.margin{4},'auto')
                  obj.usedMargin{4} = 0;
               else
                  if isreal(obj.margin{4})
                     obj.usedMargin{4} = obj.margin{4};
                  else
                     obj.usedMargin{4} = imag(obj.margin{4}) * w;
                  end
                  remain = max( 0, remain-obj.usedMargin{4} );
               end
               
               if isequal(obj.margin{2},'auto')
                  obj.usedMargin{2} = 0;
               else
                  if isreal(obj.margin{2})
                     obj.usedMargin{2} = min(remain,obj.margin{2});
                  else
                     obj.usedMargin{2} = min(remain,imag(obj.margin{2}) * w);
                  end
                  remain = max( 0, remain-obj.usedMargin{2} );
               end
               
               obj.usedWidth = remain;
               remain = 0;
               
            else
               
               % Compute width
               if isreal(obj.width)
                  obj.usedWidth = obj.width;
               else
                  obj.usedWidth = imag(obj.width) * w;
               end
               remain =  max( 0, remain - obj.usedWidth );
               
               if isequal(obj.margin{4},'auto') && isequal(obj.margin{2},'auto')
                     obj.usedMargin{4} = remain/2;
                     obj.usedMargin{2} = remain/2;
               elseif not(isequal(obj.margin{4},'auto')) && isequal(obj.margin{2},'auto')
                  if isreal(obj.margin{4})
                     obj.usedMargin{4} = obj.margin{4};
                  else
                     obj.usedMargin{4} = imag(obj.margin{4}) * w;
                  end
                  remain = max( 0, remain-obj.usedMargin{4} );
                  obj.usedMargin{2} = remain;
               elseif isequal(obj.margin{4},'auto') && not(isequal(obj.margin{2},'auto'))
                  if isreal(obj.margin{2})
                     obj.usedMargin{2} = obj.margin{2};
                  else
                     obj.usedMargin{2} = imag(obj.margin{2}) * w;
                  end
                  remain = max( 0, remain-obj.usedMargin{2} );
                  obj.usedMargin{4} = remain;
               else
                  if isreal(obj.margin{4})
                     obj.usedMargin{4} = obj.margin{4};
                  else
                     obj.usedMargin{4} = imag(obj.margin{4}) * w;
                  end
                  remain = max( 0, remain-obj.usedMargin{4} );
                  if isreal(obj.margin{2})
                     obj.usedMargin{2} = obj.margin{2};
                  else
                     obj.usedMargin{2} = imag(obj.margin{2}) * w;
                  end
                  remain = max( 0, remain-obj.usedMargin{2} );
               end
               
            end
            
         else
            
            % Inline block
            auto = obj.parent.availableAuto( obj.parent.getLineIndex(obj) );
            
            if isequal(obj.width,'auto')
               obj.usedMargin{2} = 0;
               obj.usedMargin{4} = 0;
               obj.usedWidth = auto;
               remain = max( 0, remain - auto );
            else
               
               if isreal(obj.width)
                  obj.usedWidth = obj.width;
               else
                  obj.usedWidth = imag(obj.width) * w;
               end
               remain = max( 0, remain - obj.usedWidth );
            end
               
            if isequal(obj.margin{4},'auto') && isequal(obj.margin{2},'auto')
                  obj.usedMargin{4} = remain/2;
                  obj.usedMargin{2} = remain/2;
            elseif not(isequal(obj.margin{4},'auto')) && isequal(obj.margin{2},'auto')
               if isreal(obj.margin{4})
                  obj.usedMargin{4} = obj.margin{4};
               else
                  obj.usedMargin{4} = imag(obj.margin{4}) * w;
               end
               remain = max( 0, remain-obj.usedMargin{4} );
               obj.usedMargin{2} = remain;
            elseif isequal(obj.margin{4},'auto') && not(isequal(obj.margin{2},'auto'))
               if isreal(obj.margin{2})
                  obj.usedMargin{2} = obj.margin{2};
               else
                  obj.usedMargin{2} = imag(obj.margin{2}) * w;
               end
               remain = max( 0, remain-obj.usedMargin{2} );
               obj.usedMargin{4} = remain;
            else
               if isreal(obj.margin{4})
                  obj.usedMargin{4} = obj.margin{4};
               else
                  obj.usedMargin{4} = imag(obj.margin{4}) * w;
               end
               remain = max( 0, remain-obj.usedMargin{4} );
               if isreal(obj.margin{2})
                  obj.usedMargin{2} = obj.margin{2};
               else
                  obj.usedMargin{2} = imag(obj.margin{2}) * w;
               end
               remain = max( 0, remain-obj.usedMargin{2} );
            end % if
            
         end % if
         
         obj.usedLeft = obj.left;
         obj.usedRight = obj.left + obj.usedMargin{2} + obj.usedMargin{4} ...
            + obj.usedBorderWidth(2) + obj.usedBorderWidth(4) + ...
            + obj.usedPadding(2) + obj.usedPadding(4) + obj.usedWidth;
         
      end % function
      
      
      function finalizeHeights(obj)
         
         % Compute height
         if isequal( obj.displayMode, 'block' );
            h = obj.parent.usedHeight;
            auto = obj.parent.autoHeight;
         else
            h = obj.parent.parent.usedHeight;
            index = obj.parent.getLineIndex(obj);
            auto = obj.parent.autoHeight(index);
         end
         
         % Compute padding
         if isreal(obj.padding(1))
            obj.usedPadding(1) = obj.padding(1);
         else
            obj.usedPadding(1) = imag(obj.padding(1)) * h;
         end
         if isreal(obj.padding(3))
            obj.usedPadding(3) = obj.padding(3);
         else
            obj.usedPadding(3) = imag(obj.padding(3)) * h;
         end
         
         % Compute border widths
         if isreal(obj.borderWidth(1))
            obj.usedBorderWidth(1) = obj.borderWidth(1);
         else
            obj.usedBorderWidth(1) = imag(obj.borderWidth(1))*h;
         end
         if isreal(obj.borderWidth(3))
            obj.usedBorderWidth(3) = obj.borderWidth(3);
         else
            obj.usedBorderWidth(3) = imag(obj.borderWidth(3))*h;
         end
         
         if isequal(obj.height,'auto')
            obj.usedHeight = auto;
            auto = 0;
         else
            if isreal(obj.height)
               obj.usedHeight = obj.height;
            else
               obj.usedHeight = imag(obj.height)*h;
            end
         end
         
         if isequal(obj.margin{1},'auto') && isequal(obj.margin{3},'auto')
            obj.usedMargin{1} = auto/2;
            obj.usedMargin{3} = auto/2;
            auto = 0;
         elseif not(isequal(obj.margin{1},'auto')) && isequal(obj.margin{3},'auto')
            if isreal(obj.margin{1})
               obj.usedMargin{1} = obj.margin{1};
            else
               obj.usedMargin{1} = imag(obj.margin{1})*h;
            end
            obj.usedMargin{3} = auto;
            auto = 0;
         elseif isequal(obj.margin{1},'auto') && not(isequal(obj.margin{3},'auto'))
            if isreal(obj.margin{3})
               obj.usedMargin{3} = obj.margin{3};
            else
               obj.usedMargin{3} = imag(obj.margin{3})*h;
            end
            obj.usedMargin{1} = auto;
            auto = 0;
         else
            if isreal(obj.margin{1})
               obj.usedMargin{1} = obj.margin{1};
            else
               obj.usedMargin{1} = imag(obj.margin{1})*h;
            end
            if isreal(obj.margin{3})
               obj.usedMargin{3} = obj.margin{3};
            else
               obj.usedMargin{3} = imag(obj.margin{3})*h;
            end
         end
         
         auto = obj.usedHeight;
         needsAuto = 0;
         for k = 1:obj.numChildren
            auto = auto-obj.children{k}.minHeight;
            if obj.children{k}.hasAutoHeight()
               needsAuto = needsAuto + 1;
            end
         end
         if needsAuto > 0
            obj.autoHeight = auto/needsAuto;
         else
            obj.autoHeight = 0;
         end
         
         obj.usedTop = obj.top;
         obj.usedBottom = obj.top + obj.usedMargin{1} + obj.usedMargin{3} ...
            + obj.usedBorderWidth(1) + obj.usedBorderWidth(3) + ...
            + obj.usedPadding(1) + obj.usedPadding(3) + obj.usedHeight;
         
         if not(isempty(obj.client))
            obj.client.update();
         end
         
      end % function
      
      
   end % methods
   
   
   
end % classdef