classdef LineBox < MSS.Box
   
   
   properties
      breaks
      availableWidth
      availableAuto
   end
   
   
   
   methods
      
      
%       function computeMinWidth(obj)
%          
%          obj.minWidth = 0;
%          for k = 1:obj.numChildren()
%             obj.children{k}.computeMinWidth();
%             obj.minWidth = max( obj.minWidth, obj.children{k}.minWidth );
%          end
%          
%       end % function
      

      function n = numLines(obj)
         n = length(obj.breaks)+1;
      end % function

      
      function line = getLine(obj,index)
         
         if (index > obj.numLines())
            line = {};
         end
         
         if index == 1
            left = 1;
         else
            left = obj.breaks{index-1};
         end
         
         if index == obj.numLines()
            right = obj.numChildren();
         else
            right = obj.breaks{index}-1;
         end
         
         n = obj.numChildren();
         line = obj.children(left:right);
         
      end % function
      
      
      function index = getLineIndex(obj,child)
         
         if obj.hasChild(child)
            n = obj.getChildIndex(child);
            index = 1;
            for k = 1:length(obj.breaks)
               if obj.breaks{k} <= n
                  index = index + 1;
               end
            end
         else
            index = [];
         end
         
      end % function

      
      
      function layout(obj)
         
         obj.usedWidth = obj.maxWidth;
         obj.usedHeight = obj.maxHeight;
         
         obj.computeLocation();
         obj.layoutChildren();
         
      end % function
      
      
      function layoutChildren(obj)
         
         % Compute the minHeight for each line
         for j = 1:obj.numLines()
            line = obj.getLine(j);
            maxMinHeight(j) = 0;
            countAutoH(j) = 0;
            for k = 1:length(line)
               maxMinHeight(j) = max(maxMinHeight(j),line{k}.minHeight());
               countAutoH(j) = countAutoH(j) || line{k}.countVertAuto();
            end
         end
         
         % Determine vertical auto space
         extraHeight = max(0,obj.usedHeight - sum(maxMinHeight));
         if sum(countAutoH) == 0
            autoH = 0;
         else
            autoH = extraHeight / sum(countAutoH);
         end

         for j = 1:obj.numLines()
            
            line = obj.getLine(j);
            
            % Compute the minWidth for this line
            
            totalMinWidth(j) = 0;
            for k = 1:length(line)
               totalMinWidth(j) = totalMinWidth(j) + line{k}.minWidth();
               countAutoW(j,k) = sign(line{k}.countHorzAuto());
            end
            
            % Determine horizontal auto space
            extraWidth = max(0,obj.usedWidth - totalMinWidth(j));
            if (sum(countAutoW(j,:)) == 0)
               autoW(j) = 0;
            else
               autoW(j) = extraWidth / sum(countAutoW(j,:));
            end
            
            % Compute maxHeight and maxWidth for each element
            for k = 1:length(line)
               line{k}.maxHeight = maxMinHeight(j) + autoH*countAutoH(j);
               line{k}.maxWidth = line{k}.minWidth + autoW(j)*countAutoW(j,k);
               line{k}.layout();
            end
               
         end % for j
         
        
      end % function
      
      
      function computeMinSize(obj)
         
         % Update all chidren
         for k = 1:obj.numChildren()
            obj.children{k}.computeMinSize();
         end
         
         w = 0;
         h = 0;
         for j = 1:obj.numLines()
            line = obj.getLine(j);
            lineW = 0;
            lineH = 0;
            for k = 1:length(line)
               lineW = lineW + line{k}.minWidth;
               lineH = max(lineH,line{k}.minHeight);
            end
            w = max(w,lineW);
            h = h + lineH;
         end
         
         obj.minHeight = h;
         obj.minWidth = w;

      end % function
      
      
      function n = countVertAuto(obj)
         
         n = 0;
         for j = 1:obj.numLines()
            line = obj.getLine(j);
            inThisLine = 0;
            for k = 1:length(line)
               inThisLine = inThisLine || line{k}.countVertAuto();
            end
            n = n + inThisLine;
         end
         n = double(n);
         
      end % function
      
      
      function n = countHorzAuto(obj)
         
         n = 0;
         for k = 1:obj.numChildren()
               n = n || obj.children{k}.countVertAuto();
         end
         n = double(n);
         
      end % function
      
      
      function computeMinHeight(obj)
         
         if obj.isLeaf()
            obj.minHeight = 0;
         else
            lineHeight = zeros(1,obj.numLines());
            for j = 1:obj.numLines()
               
               line = obj.getLine(j);
               for k = 1:length(line)
                  line{k}.computeMinHeight();
                  lineHeight(j) = max( lineHeight(j), line{k}.minHeight );
               end
               
            end
            obj.minHeight = sum(lineHeight);
         end
         
      end % function
      
      
      function findBreaks(obj)
         
         obj.breaks = {};
         totalW = obj.parent.width;
         accum = 0;
         for k = 1:length(obj.children)
            if accum + obj.children{k}.minWidth < totalW
               accum = accum + obj.children{k}.minWidth;
            else
               if (accum == 0) 
                  if (k < obj.numChildren())
                     obj.breaks{end+1} = k+1;
                  end
               else
                  accum = 0;
                  obj.breaks{end+1} = k;
               end
            end
         end % for k
         
      end % function
      
      
      function finalizeWidths(obj)
         
         obj.usedWidth = obj.parent.usedWidth;
         obj.usedMargin = {0, 0, 0, 0};
         obj.usedPadding = [0 0 0 0];
         obj.usedBorderWidth = [0 0 0 0];
         
         obj.availableWidth = obj.usedWidth;
         
         obj.availableAuto = zeros(1,obj.numLines());
         for j = 1:obj.numLines()
            
            totalAuto = obj.usedWidth;
            numAuto = 0;
            
            line = obj.getLine(j);
            
            for k = 1:length(line)
               totalAuto = max( 0, totalAuto - line{k}.minWidth );
               if line{k}.hasAuto()
                  numAuto = numAuto + 1;
               end
            end
            if (numAuto > 0)
               obj.availableAuto(j) = totalAuto/numAuto;
            end
            
         end % for j
         
         
      end % function
      
      function finalizeHeights(obj)
         needsAuto = false(1,obj.numLines);
         for j = 1:obj.numLines()
            line = obj.getLine(j);
            for k = 1:length(line)
               needsAuto(j) = needsAuto(j) || line{k}.hasAutoHeight();
            end
         end
         needsAuto = double(needsAuto);
         obj.autoHeight = needsAuto * obj.parent.autoHeight/ sum(needsAuto);
      end
      
      
   end % methods
   
   
   
end % classdef