classdef RootBox < MSS.Box
   
   
   
   methods
      
      
      function obj = RootBox(varargin)
         
         obj = obj@MSS.Box(varargin{:});
         
         if nargin > 0
            client = varargin{1};
            clientPosition = get(client.handle,'Position');
            obj.width = clientPosition(3);
            obj.height = clientPosition(4);
            obj.usedWidth = obj.width;
            obj.usedHeight = obj.height;
         end
         
      end % function
      
      
      function set(obj,property,value,varargin)
         
         assert(ischar(property),'Property name must be a string')
         
         switch lower(property)

            case 'display'
               
               assert( isequal(lower(property),'block'), ...
                  'The display property must be ''block''' );
               
            case 'position'
               
               assert( isequal(lower(property),'static'), ...
                  'MSS:Box:badProperty','The position property must be ''static''' );

               
            case {'width','height'}
               
               set@MSS.Box(obj,property,value)
               assert( not(isequal(obj.(lower(property)),'auto')),...
                  'Dimension %s for the RootBox cannot be ''auto''', property )
               if isnumeric(obj.(lower(property)))
                  assert( isreal(obj.(lower(property))),...
                     'Dimension %s for the RootBox cannot be a percentage', property )
               end
               obj.usedWidth = obj.width;
               obj.usedHeight = obj.height;
               
               
            case {'padding','paddingtop','paddingright','paddingbottom','paddingleft',...
                  'border','borderwidth','bordertopwidth','borderrightwidth','borderbottomwidth','borderleftwidth',...
                  'margin','margintop','marginright','marginbottom','marginleft'}
               
               error('Cannot change the padding, borders, or margins of the RootBox');
               
            otherwise
               
               set@MSS.Box(obj,property,value)
               
         end
         
         
         
         if not(isempty(varargin))
            set(obj,varargin{:});
         end
         
         obj.layout();
         
      end % function
      
      
%       function computeMinWidth(obj)
%          obj.minWidth = obj.width;
%       end % function
%       
%       function computeMinHeight(obj)
%          obj.minHeight = obj.height;
%       end % function
      
      function layout(obj)

         obj.usedTop = 0;
         obj.usedLeft = 0;
         
         obj.usedWidth = obj.width;
         obj.usedHeight = obj.height;
         
         obj.usedPadding = zeros(1,4);
         obj.usedBorderWidth = zeros(1,4);
         obj.usedMargin = zeros(1,4);
         
         obj.computeMinSize();
         obj.insertLines();

         obj.computeMinSize();
         obj.layoutChildren();
         obj.removeLines();
         obj.client.update();
         
         
         
      end % function
      
      
      function computeMinSize(obj)
         
         h = 0;
         w = 0;
         for k = 1:obj.numChildren()
            obj.children{k}.computeMinSize();
            w = w + obj.children{k}.minWidth;
            h = h + obj.children{k}.minHeight;
         end
         
         obj.minHeight = h;
         obj.minWidth = w;
         
      end % function
      

                  
      
      function layoutWidths(obj)

         % Reset the temporary dimensions
         sortedTree = obj.topologicalSort();
         for k = 1:length(sortedTree)
            sortedTree{k}.relWidth = [];
            sortedTree{k}.minWidth = [];
         end % for k
         
         % Reset the LineBoxes
         obj.removeLines();
         obj.insertLines();

         % Find breaks in LineBoxes
         sortedTree = obj.topologicalSort();
         for k = 1:length(sortedTree)
            sortedTree{k}.computeMinWidth();
         end
         for k = 1:length(sortedTree)
            if isa(sortedTree{k},'MSS.LineBox')
               sortedTree{k}.findBreaks();
            end
            sortedTree{k}.finalizeWidths();
         end

      end % function
      
      
      function layoutHeights(obj)
         
         % Reset the temporary dimensions
         sortedTree = obj.topologicalSort();
         for k = 1:length(sortedTree)
            sortedTree{k}.relHeight = [];
            sortedTree{k}.minHeight = [];
         end
         
         
         for k = 1:length(sortedTree)
            if not(isa(sortedTree{k},'MSS.LineBox'))
               sortedTree{k}.computeMinHeight();
            end
         end
         
         for k = 1:length(sortedTree)
            if isa(sortedTree{k},'MSS.LineBox')
               sortedTree{k}.computeMinHeight();
            end
         end
         
         for k = 1:length(sortedTree)
            sortedTree{k}.finalizeHeights();
         end
         
      end % function
      
      
      function finalizeWidths(obj)
         
         obj.usedWidth = obj.width;
         obj.usedMargin = {0, 0, 0, 0};
         obj.usedPadding = [0 0 0 0];
         obj.usedBorderWidth = [0 0 0 0];
         
      end % function
      
      
      function finalizeHeights(obj)
         
         obj.usedHeight = obj.height;
         obj.usedMargin = {0, 0, 0, 0};
         obj.usedPadding = [0 0 0 0];
         obj.usedBorderWidth = [0 0 0 0];
         
         auto = obj.usedHeight;
         needsAuto = 0;
         for k = 1:obj.numChildren
            auto = auto-obj.children{k}.minHeight;
            if obj.children{k}.hasAutoHeight()
               needsAuto = needsAuto + 1;
            end
         end
         if needsAuto > 0
            obj.autoHeight = auto/needsAuto;
         else
            obj.autoHeight = 0;
         end
         
      end % function
      
      
   end % methods
   
   
   
end % classdef