classdef StyleSheet < handle
   
   
   
   properties
      
      filename
      styleSheet
      parseString
      rules
      
   end % properties
   
   
   
   methods
      
      
      function obj = StyleSheet(filename)
         
         obj.styleSheet = fileread(filename);
         obj.filename = filename;
         obj.parseString = [obj.styleSheet,sprintf('\a')];
         obj.parseRules();
         
      end % function
      
      
      function obj = parseRules(obj)
         
         while not(isempty( obj.parseString ))
            selectorList = obj.getSelectorList();
            if isempty(selectorList)
               break;
            end
            selectors = obj.parseSelectorList(selectorList);
            
            blockText = obj.getBlockText();
            if isempty(blockText)
               continue;
            end
            [props,values] = obj.parseBlockText(blockText);
            
            rule = MSS.RuleSet();
            rule.addSelector(selectors(:));
            for k = 1:length(props)
               rule.addDeclaration(props{k},values{k});
            end
            
            obj.addRule(rule);
         end
         
      end % function
      
      
      function selectorList = getSelectorList(obj)
         
         [selectorList,remainder] = strtok(obj.parseString,'{');
         obj.parseString = remainder;
         
         % No need to process selectors if no rules follow
         if isempty(remainder)
            selectorList = '';
         end
         
      end % function
      
      
      function selectors = parseSelectorList(obj,selectorList)

         % Selectors are comma-separated
         rawSelectors = regexp(selectorList,'\s*,\s*','split');
         
         for k = 1:length(rawSelectors)
            % Trim whitespace from both ends
            selectorText = regexp(rawSelectors{k},'(?<=\s*)(\*|\S.*\S)(?=\s*)','match','once');
            selectors(k) = MSS.Selector(selectorText);
         end
         
      end % function
      
      
      function blockText = getBlockText(obj)
         
         [blockText,index] = regexp(obj.parseString,'(?<={\s*)\S.*?;\s*(?=})','match','once','end');
         obj.parseString(1:index+1) = [];
         
      end % function
      
      
      function [props,values] = parseBlockText(obj,blockText)
         
         declarations = regexp(blockText,'\w+\s*:\s*\S.*?\S?(?=\s*;)','match');
         props = cell(size(declarations));
         values = cell(size(declarations));
         for k = 1:length(declarations)
            props{k}  = regexp(declarations{k},'\w+(?=\s*:)','match','once');
            values{k} = regexp(declarations{k},'(?<=:\s*)\S.*\S?(?=\s*)','match','once');
         end
         
      end % function
      
      
      function obj = addRule(obj,rule)
         
         if isempty(obj.rules)
            obj.rules = rule;
         else
            obj.rules(end+1) = rule;
         end
         
      end % function
      
      
      function display(objs)
         
         for k = 1:length(objs)
            fprintf('\n%s:\n',objs(k).filename);
            display(objs(k).rules);
         end
         
      end % function
      
      
   end % methods
      
   
   
end % classdef