classdef TreeNode < handle
   
   
   
   properties (SetAccess=protected)
      parent
      children = {}
      label = ''
   end
   
   
   
   methods
      
      
      function obj = TreeNode(label)
         if nargin > 0
            obj.label = label;
         end
      end % function
      
      
      function reply = isRoot(obj)
         reply = isempty(obj.parent);
      end % function
      
      
      function reply = isLeaf(obj)
         reply = ( obj.numChildren() == 0 );
      end % function
      
      
      function root = getRoot(obj)
         if obj.isRoot()
            root = obj;
         else
            root = obj.parent().getRoot();
         end
      end % function

      
%       function obj = setParent(obj,father)
%          assert(isa(father,'MSS.TreeNode'),'Parent must be a Node object');
%          assert(isscalar(father),'A node can only have one parent');
%          if obj.isRoot()
%             obj.parent = father;
%          else
%             obj.getParent().removeChild(obj);
%             obj.parent = father;
%          end
%          father.addChild(obj);
%       end % function
      
      
      function reply = hasParent(obj,father)
         reply =  isequal(obj.parent,father);
      end % function
      
      
      function reply = hasAncestor(obj,ancestor)
         
         if obj.isRoot()
            % TODO: Can a node be it's own ancestor?
            reply = false;
         elseif obj.hasParent(ancestor)
            reply = true;
         else
            reply = obj.parent.hasAncestor(ancestor);
         end
         
      end % function
            
      
      function reply = hasChild(obj,child)
         
         reply = false;
         for k = 1:length(obj.children)
            reply = reply || isequal(child,obj.children{k});
         end
         
      end % function
      
      
      function reply = hasDescendent(obj,descendent)
         
         if obj.hasChild(descendent)
            reply = true;
         else
            reply = false;
            for k = 1:obj.numChildren()
               reply = reply || obj.children{k}.hasDescendent(descendent);
            end
         end
         
      end % function
      
      
%       function obj = removeParent(obj,father)
%          if isequal(obj.parent,father)
%             obj.parent = [];
%             father.removeChild(obj);
%          end
%       end % function
      
      
      function n = numChildren(obj)
         n = length(obj.children);
      end % function
      
      
      function index = getChildIndex(obj,child)
         if obj.hasChild(child)
            for k = 1:obj.numChildren()
               if isequal(obj.children{k},child)
                  index = k;
                  break;
               end
            end
         else
            index = [];
         end
      end % function
      
      
      function obj = addChild(obj,child)
         assert(isa(child,'MSS.TreeNode'),'Children must be node objects');
         if not(obj.hasChild(child))
            assert(child.isRoot(),'This node already has a parent');
            obj.children{end+1} = child;
            if not(child.isRoot())
               child.parent.removeChild(child);
            end
            child.parent = obj;
         end
      end % function
      
      
      function obj = addChildren(obj,varargin)
         for k = 1:length(varargin)
            obj.addChild(varargin{k});
         end
      end % function
      
      
      function obj = insertChild(obj,child,index)
         
         assert(index >= 1,' Index out of bounds');
         assert(index <= obj.numChildren+1,'Index out of bounds');
         
         % Insert child into children array
         obj.children = [obj.children(1:index-1),{child},obj.children(index:end)];
         
         % Set new child's parent
         if not(child.isRoot())
            child.parent.removeChild(child);
         end
         child.parent = obj;
            
      end % function
      
      
      function obj = removeChild(obj,child)
         if obj.hasChild(child)
            index = obj.getChildIndex(child);
            obj.children(index) = [];
            child.parent = [];
         end
      end % function
      
      
      function obj = insertGeneration(obj,new,left,right)
         
         if nargin == 2
            right = left;
         end
         assert( not(isempty(obj.children)), 'Node must have children' );
         assert( left >= 1, 'Left index out of bounds' );
         assert( right <= obj.numChildren(), 'Right index out of bounds' );

         orphans = obj.children(left:right);
         obj.insertChild(new,left);
         for k = 1:length(orphans)
            obj.removeChild(orphans{k});
            new.addChild(orphans{k});
         end
                     
      end % function
      
      
      function obj = removeGeneration(obj,target)
         
         assert( obj.hasChild(target), 'One node must a child of the other' );
         
         index = obj.getChildIndex(target);
         obj.removeChild(target);
         refugees = target.children;
         for k = length(refugees):-1:1
            target.removeChild(refugees{k});
            obj.insertChild(refugees{k},index);
         end
         
      end % function
      
      
      
      function siblings = getSiblings(obj)
         siblings = obj.parent.children;
         index = obj.parent.getChildIndex(obj);
         siblings(index) = [];
      end % function
      
      
      function prevSibling = getPrevSibling(obj)
         siblings = obj.getSiblings();
         index = obj.parent.getChildIndex(obj);
         if obj.isRoot() || (index == 1) || isempty(siblings)
            prevSibling = [];
         else
            prevSibling = siblings{index-1};
         end
      end % function
      
      
      function nextSibling = getNextSibling(obj)
         siblings = obj.getSiblings();
         index = obj.parent.getChildIndex(obj);
         if obj.isRoot() || (index > length(siblings)) || isempty(siblings)
            nextSibling = [];
         else
            nextSibling = siblings{index};
         end
      end % function
      
      
      function nodes = topologicalSort(obj)
         nodes = {obj};
         for k = 1:obj.numChildren()
            nodes = [nodes,obj.children{k}.topologicalSort()];
         end
      end % function
      
     
   end % methods
   
   
   
end % classdef