classdef axes < MSS.Element

   properties
      handle
      box
      grid
   end
   
   methods

      
      function obj = axes(h)
         
         obj = obj@MSS.Element('axes');
         fig = MSS.gcf();
         if (nargin > 0) && ishandle(h)
            if isequal(get(h,'Type'),'axes')
               obj.handle = h;
            else
               error('Input must be an axes handle')
            end
         else
            obj.handle = builtin('axes','Units','pixels');
         end
         obj.box = MSS.FrameBox(obj);
         
         fig.addChild(obj);
         fig.box.addChild(obj.box);
         
         xLabelHandle = builtin('text');
         yLabelHandle = builtin('text');
         set(obj.handle,'XLabel',xLabelHandle);
         set(obj.handle,'YLabel',yLabelHandle);
         xLab = MSS.text(xLabelHandle);
         yLab = MSS.text(yLabelHandle);
         obj.addChild(xLab);
         obj.addChild(yLab);
         
         obj.grid = MSS.grid();
         obj.addChild(obj.grid);
         
         outer = get(obj.handle,'OuterPosition');
         tight = get(obj.handle,'TightInset');
         pos = get(obj.handle,'Position');
         
         set( obj.box, 'Width', pos(3)+tight(1)+tight(3) );
         set( obj.box, 'Height', pos(4)+tight(2)+tight(4) );
%          set( obj.box, 'Width', 'auto' );
%          set( obj.box, 'Height', 'auto' );
         
         set( obj.box, 'PaddingTop', outer(4)-(pos(2)+pos(4)+tight(4)) );
         set( obj.box, 'PaddingRight', outer(3)-(pos(1)+pos(3)+tight(3)) );
         set( obj.box, 'PaddingBottom', pos(2)-tight(2) );
         set( obj.box, 'PaddingLeft', pos(1)-tight(1) );
         
         set( obj.box, 'margin', 'auto' );
         
         fig.layout();
         
      end % function
      
      
      function set(obj,property,value,varargin)
         
         boxChanged = false;
         switch lower(property)
            case {'color','xcolor','ycolor'}
               color = MSS.Converter.standardizeColors(value);
               assert(isscalar(color));
               set(obj.handle,property,color{1});
               
            case {'fontsize'}
               value = MSS.Converter.standardizeLengths(value);
               fontUnits = get(obj.handle,'FontUnits');
               set(obj.handle,'FontUnits','pixels')
               set(obj.handle,'FontSize',value{1})
               set(obj.handle,'FontUnits',fontUnits)
               
            otherwise
               try
                  set(obj.box,property,value);
                  boxChanged = true;
               catch except
                  if isequal(except.identifier,'MSS:Box:badProperty')
                     try
                        if ismember(lower(property),{'color','xcolor','ycolor'})
                           throw(MException('MSS:grid:badProperty','Hack'))
                        else
                           set(obj.grid,property,value)
                        end
                     catch except
                        if isequal(except.identifier,'MSS:grid:badProperty')
                           set(obj.handle,property,value);
                        end
                     end
                     boxChanged = false;
                  else
                     except.rethrow();
                  end
               end
         end
         
         if boxChanged
%             obj.box.getRoot().layout();
         end
         
         if not(isempty(varargin))
            set(obj,varargin{:})
         end
         
%          obj.update();
         
      end % function
      
      
      function value = get(obj,property,value)
         
         try
            switch lower(property)
               case 'xlabel'
                  value = obj.children{1};
               case 'ylabel'
                  value = obj.children{2};
               otherwise
               value = get(obj.handle,property);
            end
         catch
            try
               value = get(obj.grid,property);
            catch
               value = get(obj.box,property);
            end
         end
         
      end % function
      
      
      function update(obj)
         
         h = get(obj.getRoot(),'Height');
         tight = get(obj.handle,'TightInset');
         pos = zeros(1,4);
         pos(3) = obj.box.usedWidth - tight(1) - tight(3);
         pos(4) = obj.box.usedHeight - tight(2) - tight(4);
         pos(2) = h-obj.box.usedTop-obj.box.usedMargin(1)-obj.box.usedBorderWidth(1)-obj.box.usedPadding(1)-obj.box.usedHeight+tight(2);
         pos(1) = obj.box.usedLeft + obj.box.usedBorderWidth(4) + obj.box.usedPadding(4) + tight(1);

         set(obj.handle,'Position',pos);
         
         for k = 1:obj.numChildren()
            obj.children{k}.update();
         end
         
      end % function
      
      
      function delete(obj)
         if ishandle(obj.handle)
            if not(isequal(get(obj.handle,'BeingDeleted'),'on'))
               delete(obj.handle);
            end
         end
         obj.parent.removeChild(obj);
         obj.box.parent.removeChild(obj.box);
         for child = obj.children
            if isvalid(child{1})
               delete(child{1});
            end
         end
      end % function
      
      
   end % methods
   
   
   
end % classdef

