classdef figure < MSS.Element



   properties

      box
      handle

   end % properties



   methods


      function obj = figure(varargin)

         obj = obj@MSS.Element('figure');
         obj.handle = builtin('figure',varargin{:});
         obj.box = MSS.RootBox(obj);
         
%          set(obj.handle,'ResizeFcn',@(s,e)resize(obj,s,e));
         set(obj.handle,'DeleteFcn',@(s,e)deleteCallback(obj,s,e));

         guiData.owner = obj;
         guidata(obj.handle,guiData);

         obj.box.layout();

      end % function


      function set(obj,property,value,varargin)
         
         switch lower(property)
            
            case 'color'
               color = MSS.Converter.standardizeColors(value);
               assert( isscalar(color) );
               set(obj.handle,'Color',color{:});
               
            otherwise
               
               try
                  set(obj.box,property,value);
                  
               catch except
                  
                  if not(isequal(except.identifier,'MSS:Box:badProperty'))
                     except.rethrow()
                  end
                  % Pass on to the hg figure
                  set(obj.handle,property,value);
                  
                  % Update the RootBox if necessary
                  if ismember( lower(property), {'position'} )
                     obj.layout();
                  end
                  
               end % try-catch
               
         end % switch

         if not(isempty(varargin))
            set(obj,varargin{:})
         end
         
      end % function set


      function value = get(obj,property)

         try
            value = get(obj.handle,property);
            
            if isequal(lower(property),'currentaxes')
               for k = 1:obj.numChildren()
                  if isa(obj.children{k},'MSS.axes') && (value == obj.children{k}.handle)
                     value = obj.children{k};
                  end
               end
            end
            
         catch
            value = get(obj.box,property);
         end

      end % function get

      
      function layout(obj)
         obj.box.layout();
      end % function
      
      
      function apply(obj,styleSheet)
         
         if ischar(styleSheet)
            styleSheet = MSS.StyleSheet(styleSheet);
         end
         
         for rule = styleSheet.rules
            
            % Get matching elements
            elements = {};
            for select = rule.selectors
               for j = 1:length(select)
                  elements = [elements,select(j).selectElements(obj)];
               end
            end
            
            % Try to apply this rule to each element.
            for j = 1:length(elements)
               for k = 1:length(rule.props)
                  try
                     set(elements{j},rule.props{k},rule.values{k})
                  catch except
                     % Write error to a log file?
                     disp(except.message);
                  end
               end
            end
         end
         
         obj.layout();
            
      end % function
      

      function update(obj)

         offset = 20;
         screen = get(0,'ScreenSize');
         pos = get(obj.handle,'Position');
         pos(3) = obj.box.width;
         pos(4) = obj.box.height;
         if pos(4) > screen(4) - offset;
            pos(2) = screen(4)-pos(4)-offset;
         else
            pos(2) = (screen(4)-pos(4))/2-offset;
         end
         pos(1) = (screen(3)-pos(3))/2;
         set(obj.handle,'Position',pos)
         
         actualPos = get(obj.handle,'Position');
         assert(isequal(pos,actualPos),'Update failed.')
         
         for child = obj.children
            child{1}.update();
         end

      end

      function resize(obj,source,event)
         pos = get(obj.handle,'Position');
         set(obj.box,'Width',pos(3),'Height',pos(4));
         obj.box.layout();
      end % function
      
      
      function close(obj)
         close(obj.handle);
      end


      function deleteCallback(obj,source,event)
         if isvalid(obj)
            obj.delete();
         end
      end


      function delete(obj)
         if ishandle(obj.handle)
            if not(isequal(get(obj.handle,'BeingDeleted'),'on'))
               close(obj.handle);
            end
         end
         for child = obj.children
            delete(child{1});
         end
      end



   end % methods



end % classdef

