classdef grid < MSS.Element
   
   properties
      handle
      offset = 0.01
      gridType = 'major'
      xGrid = 'on'
      yGrid = 'on'
      xColor = 0.8*[1 1 1]
      yColor = 0.8*[1 1 1]
      xStyle = '-'
      yStyle = '-'
      xLineWidth = 0.5
      yLineWidth = 0.5
   end
   
   methods
      
      function obj = grid()
         
         obj = obj@MSS.Element('grid');
      
      end
      
      function update(obj)
         
         for k = 1:length(obj.handle)
            if ishandle(obj.handle(k))
               delete(obj.handle(k));
            end
         end
         obj.handle = [];
         
         a = obj.parent;
         
         existingChildren = get(a.handle,'Children');
         nextPlot = get(a.handle,'NextPlot');
         set(a.handle,'NextPlot','add');
         
         xTick = get(a.handle,'XTick');
         yTick = get(a.handle,'YTick');
         xLim = get(a.handle,'XLim');
         yLim = get(a.handle,'YLim');
         xLimMode = get(a.handle,'XLimMode');
         yLimMode = get(a.handle,'XLimMode');
         xColor = get(a.handle,'XColor');
         yColor = get(a.handle,'YColor');
         color = get(a.handle,'Color');
         
         set(a.handle,'XLimMode','manual')
         set(a.handle,'YLimMode','manual')
         
         handles = [];
         if isequal(obj.xGrid,'on')
            if isequal(get(a.handle,'Box'),'on')
               yCoords = yLim+obj.offset*[1,-1].*abs(diff(yLim));
            else
               yCoords = yLim+obj.offset*[1,0].*abs(diff(yLim));
            end
            for k = 1:length(xTick)
               if (xTick(k) == xLim(1))
                  handles(end+1) = plot(a.handle,xTick(k)*[1 1],yCoords,...
                     obj.xStyle,'Color',yColor,'LineWidth',obj.xLineWidth);
               elseif (xTick(k) == xLim(2))
                  if isequal(get(a.handle,'Box'),'on')
                     handles(end+1) = plot(a.handle,xTick(k)*[1 1],yCoords,...
                        obj.xStyle,'Color',yColor,'LineWidth',obj.xLineWidth);
                  else
                     handles(end+1) = plot(a.handle,xTick(k)*[1 1],yCoords,...
                        obj.xStyle,'Color',color,'LineWidth',obj.xLineWidth);
                  end
               else
                  handles(end+1) = plot(a.handle,xTick(k)*[1 1],yCoords,...
                     obj.xStyle,'Color',obj.xColor,'LineWidth',obj.xLineWidth);
               end
            end
         end
         if isequal(obj.yGrid,'on')
            if isequal(get(a.handle,'Box'),'on')
               xCoords = xLim+obj.offset*[1,-1].*abs(diff(xLim));
            else
               xCoords = xLim+obj.offset*[1,0].*abs(diff(xLim));
            end
            for k = 1:length(yTick)
               if (yTick(k) == yLim(1))
                  handles(end+1) = plot(a.handle,xCoords,yTick(k)*[1 1],...
                     obj.yStyle,'Color',xColor,'LineWidth',obj.yLineWidth);
               elseif (yTick(k) == yLim(2))
                  if isequal(get(a.handle,'Box'),'on')
                     handles(end+1) = plot(a.handle,xCoords,yTick(k)*[1 1],...
                        obj.yStyle,'Color',xColor,'LineWidth',obj.yLineWidth);
                  else
                     handles(end+1) = plot(a.handle,xCoords,yTick(k)*[1 1],...
                        obj.yStyle,'Color',color,'LineWidth',obj.yLineWidth);
                  end
               else
                  handles(end+1) = plot(a.handle,xCoords,yTick(k)*[1 1],...
                     obj.yStyle,'Color',obj.yColor,'LineWidth',obj.yLineWidth);
               end
            end
         end
         obj.handle = handles;
         
         set(a.handle,'children',[existingChildren,handles]);
         set(a.handle,'NextPlot',nextPlot);
         
      end
      
      
      function set(obj,property,value,varargin)
         
         switch lower(property)
            
            case 'xgrid'
               if ismember(lower(value),{'off','on'})
                  obj.xGrid = lower(value);
               else
                  error('XGrid must be ''off'' or ''on''')
               end
               
            case 'ygrid'
               if ismember(lower(value),{'off','on'})
                  obj.yGrid = lower(value);
               else
                  error('YGrid must be ''off'' or ''on''')
               end
               
            case 'xcolor'
               color = MSS.Converter.standardizeColors(value);
               assert( isscalar(color), 'Too many colors' );
               obj.xColor = color{1};
               
            case 'ycolor'
               color = MSS.Converter.standardizeColors(value);
               assert( isscalar(color), 'Too many colors' );
               obj.yColor = color{1};
               
            case 'color'
               color = MSS.Converter.standardizeColors(value);
               assert( isscalar(color), 'Too many colors' );
               obj.xColor = color{1};
               obj.yColor = color{1};
               
            case 'xlinestyle'
               if ismember( value, {'-','--',':','-.'} )
                  obj.xStyle = value;
               else
                  error('Invalid line style')
               end
               
            case 'ylinestyle'
               if ismember( value, {'-','--',':','-.'} )
                  obj.yStyle = value;
               else
                  error('Invalid line style')
               end
               
            case 'linestyle'
               if ismember( value, {'-','--',':','-.'} )
                  obj.xStyle = value;
                  obj.yStyle = value;
               else
                  error('Invalid line style')
               end
               
            case 'xlinewidth'
               value = MSS.Converter.standardizeLengths(value);
               assert( isnumeric(value{1}) )
               assert( isscalar(value) )
               assert( value{1} > 0 )
               obj.xLineWidth = value{1};
               
            case 'ylinewidth'
               value = MSS.Converter.standardizeLengths(value);
               assert( isnumeric(value{1}) )
               assert( isscalar(value) )
               assert( value{1} > 0 )
               obj.yLineWidth = value{1};
               
            case 'linewidth'
               value = MSS.Converter.standardizeLengths(value);
               assert( isnumeric(value{1}) )
               assert( isscalar(value) )
               assert( value{1} > 0 )
               obj.xLineWidth = value{1};
               obj.yLineWidth = value{1};
               
            otherwise
               
               throw(MException('MSS:grid:badProperty','Invalid property'));

         end
         
         if not(isempty(varargin))
            set(obj,varargin{:});
         else
            obj.update();
         end
         
      end % function set 
      
   end
end